package ink.wgink.module.file.components;

import ink.wgink.exceptions.FileException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.exceptions.base.SystemException;
import ink.wgink.module.file.enums.UploadTypeEnum;
import ink.wgink.module.file.service.IFileService;
import ink.wgink.properties.FileProperties;
import ink.wgink.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * @ClassName: FileUtil
 * @Description: 文件工具
 * @Author: wanggeng
 * @Date: 2022/7/25 17:00
 * @Version: 1.0
 */
@Component
public class FileComponent {

    protected static final Logger LOG = LoggerFactory.getLogger(FileComponent.class);
    private static final char[] HEX_CODE = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
    private String[] imageTypes;
    private String[] videoTypes;
    private String[] audioTypes;
    private String[] fileTypes;
    @Autowired
    private FileProperties fileProperties;

    /**
     * 保存文件
     *
     * @param fileInputStream
     * @param fileName
     * @param filePath
     * @return
     */
    public long saveFile(InputStream fileInputStream, String fileName, String filePath) {
        File uploadFolder = new File(filePath);
        if (!uploadFolder.exists()) {
            uploadFolder.mkdirs();
        }
        FileOutputStream uploadFileOutputStream = null;
        long fileSize = 0;
        try {
            uploadFileOutputStream = new FileOutputStream(uploadFolder + File.separator + fileName);
            int readLength;
            for (byte[] buf = new byte[IFileService.INPUT_STREAM_SIZE]; (readLength = fileInputStream.read(buf)) > -1; ) {
                uploadFileOutputStream.write(buf, 0, readLength);
                fileSize += readLength;
            }
            uploadFileOutputStream.flush();
        } catch (Exception e) {
            LOG.error(e.getMessage(), e);
            throw new FileException("文件上传失败");
        } finally {
            try {
                if (null != uploadFileOutputStream) {
                    uploadFileOutputStream.close();
                }
                if (null != fileInputStream) {
                    fileInputStream.close();
                }
            } catch (Exception e1) {
                LOG.error(e1.getMessage());
                throw new FileException("文件上传失败");
            }
        }
        return fileSize;
    }

    /**
     * 删除源文件
     *
     * @param sourceFilePath
     */
    public static void deleteSourceFile(String sourceFilePath) {
        File file = new File(sourceFilePath);
        if (file.exists()) {
            boolean isDelete = file.delete();
            if (isDelete) {
                LOG.debug("文件删除成功");
            } else {
                LOG.debug("文件删除失败");
            }
        }
    }

    /**
     * 获取文件的MD5
     *
     * @param file
     * @return
     */
    public String getFileMD5(File file) {
        if (file == null) {
            throw new SearchException("文件不存在");
        }
        if (!file.exists()) {
            throw new SearchException("文件不存在");
        }
        String fileMd5;
        try (InputStream inputStream = Files.newInputStream(file.toPath())) {
            MessageDigest messageDigest = MessageDigest.getInstance("MD5");
            int readLength;
            for (byte[] buf = new byte[IFileService.INPUT_STREAM_SIZE]; (readLength = inputStream.read(buf)) > -1; ) {
                messageDigest.update(buf, 0, readLength);
            }
            // 计算文件的MD5
            byte[] data = messageDigest.digest();
            StringBuilder fileMd5SB = new StringBuilder(data.length * 2);
            for (byte b : data) {
                fileMd5SB.append(HEX_CODE[(b >> 4) & 0xF]);
                fileMd5SB.append(HEX_CODE[(b & 0xF)]);
            }
            fileMd5 = fileMd5SB.toString();
        } catch (IOException | NoSuchAlgorithmException e) {
            throw new SystemException(e);
        }
        return fileMd5;
    }

    /**
     * 获取上传绝对文件
     *
     * @param uploadTypeEnum
     * @param fileType
     * @return
     * @throws FileException
     */
    public String getUploadPath(UploadTypeEnum uploadTypeEnum, String fileType) throws FileException {
        String baseUploadPath = fileProperties.getUploadPath();
        if (StringUtils.isBlank(baseUploadPath)) {
            throw new SystemException("上传路径未配置");
        }
        StringBuilder filePath = new StringBuilder();
        if (!baseUploadPath.endsWith(File.separator)) {
            filePath.append(baseUploadPath).append(File.separator);
        } else {
            filePath.append(baseUploadPath);
        }
        boolean hasFileType = !StringUtils.isBlank(fileType);
        if (uploadTypeEnum.getValue() == UploadTypeEnum.IMAGE.getValue()) {
            if (hasFileType && !isTypeCorrect(getImageTypes(), fileType)) {
                throw new FileException("图片格式不支持上传");
            }
            filePath.append("images");
        } else if (uploadTypeEnum.getValue() == UploadTypeEnum.VIDEO.getValue()) {
            if (hasFileType && !isTypeCorrect(getVideoTypes(), fileType)) {
                throw new FileException("视频格式不支持上传");
            }
            filePath.append("videos");
        } else if (uploadTypeEnum.getValue() == UploadTypeEnum.AUDIO.getValue()) {
            if (hasFileType && !isTypeCorrect(getAudioTypes(), fileType)) {
                throw new FileException("音频格式不支持上传");
            }
            filePath.append("audios");
        } else if (uploadTypeEnum.getValue() == UploadTypeEnum.ERROR_EXCEL.getValue()) {
            filePath.append("errorexcels");
        } else {
            if (hasFileType && !isTypeCorrect(getFileTypes(), fileType)) {
                throw new FileException("文件格式不支持上传");
            }
            filePath.append("files");
        }
        filePath.append(File.separator).append(DateUtil.getDays());
        return filePath.toString();
    }

    /**
     * 获取文件url
     *
     * @param uploadPath
     * @return
     */
    public String getFileUrl(String uploadPath) {
        String fileUrl = uploadPath.replace(fileProperties.getUploadPath(), "");
        if ("\\".equals(File.separator)) {
            fileUrl = fileUrl.replace("\\", "/");
        }
        if (fileUrl.startsWith("/")) {
            fileUrl = fileUrl.substring(1, fileUrl.length() - 1);
        }
        return fileUrl;
    }

    /**
     * 获取文件类型
     *
     * @param fileName
     * @return
     */
    public String getFileType(String fileName) {
        String[] names = fileName.split("\\.");
        if (names != null) {
            return names[names.length - 1].toLowerCase();
        }
        return "";
    }

    /**
     * 校验类型
     *
     * @param types
     * @param fileType
     * @return
     */
    public boolean isTypeCorrect(String[] types, String fileType) {
        for (String type : types) {
            if (StringUtils.equalsIgnoreCase(fileType, type)) {
                return true;
            }
        }
        return false;
    }

    public String[] getImageTypes() {
        return imageTypes == null ? fileProperties.getImageTypes().split(",") : imageTypes;
    }

    public String[] getVideoTypes() {
        return videoTypes == null ? fileProperties.getVideoTypes().split(",") : videoTypes;
    }

    public String[] getAudioTypes() {
        return audioTypes == null ? fileProperties.getAudioTypes().split(",") : audioTypes;
    }

    public String[] getFileTypes() {
        return fileTypes == null ? fileProperties.getFileTypes().split(",") : fileTypes;
    }

}
