package ink.wgink.module.file.service.v2.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.SearchException;
import ink.wgink.module.file.components.FileComponent;
import ink.wgink.module.file.dao.IFileDao;
import ink.wgink.module.file.enums.UploadTypeEnum;
import ink.wgink.module.file.pojo.dtos.v2.FileUploadSuccessDTO;
import ink.wgink.module.file.pojo.vos.v2.FileSaveVO;
import ink.wgink.module.file.pojo.vos.v2.FileUpdateVO;
import ink.wgink.module.file.service.v2.IFileService;
import ink.wgink.pojo.pos.FilePO;
import ink.wgink.util.UUIDUtil;
import ink.wgink.util.date.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Map;

/**
 * @ClassName: FileV2ServiceImpl
 * @Description: 文件V2
 * @Author: wanggeng
 * @Date: 2022/4/13 16:15
 * @Version: 1.0
 */
@Service("fileServiceV2")
public class FileServiceImpl extends DefaultBaseService implements IFileService {

    @Autowired
    private FileComponent fileComponent;
    @Autowired
    private IFileDao fileDao;

    @Override
    public FileUploadSuccessDTO uploadSingle(MultipartFile uploadFile, UploadTypeEnum uploadTypeEnum) {
        FileSaveVO fileSaveVO = getFileSaveVO(uploadFile, uploadTypeEnum);
        fileSaveVO.setCreator(securityComponent.getCurrentUser().getUserId());
        return saveFile(fileSaveVO);
    }

    @Override
    public FileUploadSuccessDTO uploadSingle(InputStream inputStream, UploadTypeEnum uploadTypeEnum, String fileName) {
        FileSaveVO fileSaveVO = getFileSaveVO(inputStream, uploadTypeEnum, fileName);
        fileSaveVO.setCreator(securityComponent.getCurrentUser().getUserId());
        return saveFile(fileSaveVO);
    }


    @Override
    public FileUploadSuccessDTO uploadSingleByToken(String token, MultipartFile uploadFile, UploadTypeEnum uploadTypeEnum) {
        FileSaveVO fileSaveVO = getFileSaveVO(uploadFile, uploadTypeEnum);
        fileSaveVO.setCreator(getAppTokenUser(token).getId());
        return saveFile(fileSaveVO);
    }

    @Override
    public FileUploadSuccessDTO uploadSingleByToken(String token, InputStream inputStream, UploadTypeEnum uploadTypeEnum, String fileName) {
        FileSaveVO fileSaveVO = getFileSaveVO(inputStream, uploadTypeEnum, fileName);
        fileSaveVO.setCreator(getAppTokenUser(token).getId());
        return saveFile(fileSaveVO);
    }

    @Override
    public FileUploadSuccessDTO uploadSingleByUserId(String userId, MultipartFile uploadFile, UploadTypeEnum uploadTypeEnum) {
        FileSaveVO fileSaveVO = getFileSaveVO(uploadFile, uploadTypeEnum);
        fileSaveVO.setCreator(userId);
        return saveFile(fileSaveVO);
    }

    @Override
    public FileUploadSuccessDTO uploadSingleByUserId(String userId, InputStream inputStream, UploadTypeEnum uploadTypeEnum, String fileName) {
        FileSaveVO fileSaveVO = getFileSaveVO(inputStream, uploadTypeEnum, fileName);
        fileSaveVO.setCreator(userId);
        return saveFile(fileSaveVO);
    }

    @Override
    public FileUploadSuccessDTO saveFile(FileSaveVO fileSaveVO) {
        String fileId = UUIDUtil.getUUID();
        Map<String, Object> params = getHashMap(10);
        params.put("fileId", fileId);
        params.put("fileName", fileSaveVO.getFileName());
        params.put("filePath", fileSaveVO.getUploadPath());
        params.put("fileUrl", fileComponent.getFileUrl(fileSaveVO.getUploadPath()));
        params.put("fileType", fileSaveVO.getFileType());
        params.put("fileSize", fileSaveVO.getFileSize());
        params.put("isBack", 0);

        String time = DateUtil.getTime();
        params.put("creator", fileSaveVO.getCreator());
        params.put("modifier", fileSaveVO.getCreator());
        params.put("gmtCreate", time);
        params.put("gmtModified", time);
        params.put("isDelete", 0);
        fileDao.save(params);

        return new FileUploadSuccessDTO(fileId, fileSaveVO.getFileName(), fileSaveVO.getFileSize());
    }

    @Override
    public void update(FileUpdateVO fileUpdateVO) {
        Map<String, Object> params = getHashMap(10);
        params.put("fileId", fileUpdateVO.getFileId());
        params.put("fileName", fileUpdateVO.getFileName());
        params.put("filePath", fileUpdateVO.getUploadPath());
        params.put("fileUrl", fileComponent.getFileUrl(fileUpdateVO.getUploadPath()));
        params.put("fileType", fileUpdateVO.getFileType());
        params.put("fileSize", fileUpdateVO.getFileSize());
        params.put("modifier", fileUpdateVO.getCreator());
        params.put("gmtModified", DateUtil.getTime());
        fileDao.update(params);
    }

    @Override
    public FilePO getPO(Map<String, Object> params) {
        return fileDao.getPO(params);
    }

    @Override
    public FilePO getPO(String fileId) {
        Map<String, Object> params = getHashMap(2);
        params.put("fileId", fileId);
        return getPO(params);
    }

    /**
     * 获取文件保存VO
     *
     * @param uploadFile
     * @param uploadTypeEnum
     * @return
     */
    private FileSaveVO getFileSaveVO(MultipartFile uploadFile, UploadTypeEnum uploadTypeEnum) {
        String fileName = uploadFile.getOriginalFilename();
        InputStream inputStream;
        try {
            inputStream = uploadFile.getInputStream();
        } catch (IOException e) {
            throw new SearchException(e);
        }
        return getFileSaveVO(inputStream, uploadTypeEnum, fileName);
    }

    /**
     * 获取文件保存VO
     *
     * @param inputStream
     * @param uploadTypeEnum
     * @param fileName
     * @return
     */
    private FileSaveVO getFileSaveVO(InputStream inputStream, UploadTypeEnum uploadTypeEnum, String fileName) {
        String fileType = fileComponent.getFileType(fileName);
        String uploadPath = fileComponent.getUploadPath(uploadTypeEnum, fileType);
        String uuidFileName = UUIDUtil.get32UUID() + "." + fileType;
        long fileSize = fileComponent.saveFile(inputStream, uuidFileName, uploadPath);
        // 保存文件
        FileSaveVO fileSaveVO = new FileSaveVO();
        fileSaveVO.setFileName(fileName);
        fileSaveVO.setFileType(fileType);
        fileSaveVO.setUploadPath(uploadPath + File.separator + uuidFileName);
        fileSaveVO.setFileSize(fileSize);
        return fileSaveVO;
    }

}
